package org.androidtransfuse.gen.componentBuilder;

import com.google.inject.assistedinject.Assisted;
import com.sun.codemodel.JBlock;
import com.sun.codemodel.JClassAlreadyExistsException;
import com.sun.codemodel.JDefinedClass;
import com.sun.codemodel.JExpression;
import org.androidtransfuse.gen.InjectionFragmentGenerator;
import org.androidtransfuse.layout.LayoutHandlerDelegate;
import org.androidtransfuse.model.InjectionNode;
import org.androidtransfuse.model.TypedExpression;
import org.androidtransfuse.util.Logger;

import javax.inject.Inject;
import java.util.Map;

/**
 * @author John Ericksen
 */
public class LayoutHandlerBuilder implements LayoutBuilder {

    private final InjectionFragmentGenerator injectionFragmentGenerator;
    private final InjectionNode layoutHandlerInjectionNode;
    private final Logger logger;

    @Inject
    public LayoutHandlerBuilder(InjectionFragmentGenerator injectionFragmentGenerator,
                                @Assisted InjectionNode layoutHandlerInjectionNode,
                                Logger logger) {
        this.injectionFragmentGenerator = injectionFragmentGenerator;
        this.layoutHandlerInjectionNode = layoutHandlerInjectionNode;
        this.logger = logger;
    }

    @Override
    public void buildLayoutCall(JDefinedClass definedClass, JBlock block) {

        try {
            Map<InjectionNode, TypedExpression> expressionMap = injectionFragmentGenerator.buildFragment(block, definedClass, layoutHandlerInjectionNode);

            //LayoutHandlerDelegate.invokeLayout()
            JExpression layoutHandlerDelegate = expressionMap.get(layoutHandlerInjectionNode).getExpression();

            block.add(layoutHandlerDelegate.invoke(LayoutHandlerDelegate.INVOKE_LAYOUT_METHOD));

        } catch (ClassNotFoundException e) {
            logger.error("ClassNotFoundExcetion while trying to generate LayoutHandler", e);
        } catch (JClassAlreadyExistsException e) {
            logger.error("JClassAlreadyExistsException while trying to generate LayoutHandler", e);
        }
    }
}
